#!/usr/bin/env pybricks-micropython
from pybricks.hubs import EV3Brick
from pybricks.ev3devices import Motor, TouchSensor, ColorSensor                              
from pybricks.parameters import Port, Stop, Direction, Button, Color
from pybricks.tools import wait, StopWatch, DataLog
from pybricks.messaging import BluetoothMailboxServer, TextMailbox
from pybricks.robotics import DriveBase

# This program requires LEGO EV3 MicroPython v2.0 or higher.
# Click "Open user guide" on the EV3 extension tab for more information.

# LEGO Spinner Factory - Bridge Control
# 
# Created from scratch by Adam and Alex
# Visit our web site for detal - molex.bversion.com

# Create your objects here.
ev3 = EV3Brick()
left_wheel = Motor(Port.D, Direction.COUNTERCLOCKWISE)
right_wheel = Motor(Port.B, Direction.COUNTERCLOCKWISE)
start_button = TouchSensor(Port.S1)
rail_detector = ColorSensor(Port.S2)
color_selector = ColorSensor(Port.S3)

# Connect to the other EV3 for head control
# ev3.speaker.say("Start the head controller now")
ev3.speaker.beep()
server = BluetoothMailboxServer()
mbox = TextMailbox('Factory', server)
server.wait_for_connection()

# Define the functions
def rail_color_detect():# 1 - White, 2 - Yellow, 3 - Blue, 4 - Green, 5 - Red, 0 - unstable, 99 - others
    for i in range(0,300):
        if i == 0:
            first_color = rail_detector.color()
        if rail_detector.color() != first_color:
            return 0
    if first_color == Color.WHITE:
        return 1
    if first_color == Color.YELLOW:
        return 2
    if first_color == Color.BLUE:
        return 3
    if first_color == Color.GREEN:
        return 4
    if first_color == Color.RED:
        return 5
    return 99

def color_selector_detect():# 1 - White, 2 - Yellow, 3 - Blue, 4 - Green, 5 - Red, 0 - unstable, 99 - others
    for i in range(0,300):
        if i == 0:
            first_color = color_selector.color()
        if color_selector.color() != first_color:
            return 0
    if first_color == Color.WHITE:
        return 1
    if first_color == Color.YELLOW:
        return 2
    if first_color == Color.BLUE:
        return 3
    if first_color == Color.GREEN:
        return 4
    if first_color == Color.RED:
        return 5
    return 99    
     
def color_selection(): 
    i = 0
    yellow = 0
    blue = 0
    green = 0
    red = 0

    while i < 4:
        color = color_selector_detect()
        if color == 2 and yellow == 0:
            color_seq.append(color)
            yellow = 1
            i += 1
            ev3.speaker.say("Yellow")            

        if color == 3 and blue == 0:
            color_seq.append(color)
            blue = 1
            i += 1
            ev3.speaker.say("Blue")
            
        if color == 4 and green == 0:
            color_seq.append(color)
            green = 1
            i += 1
            ev3.speaker.say("Green")
            
        if color == 5 and red == 0:
            color_seq.append(color)
            red = 1
            i += 1
            ev3.speaker.say("Red")
    
def bridge_position():
    bridge_move(-180)
    
    while True:
        if rail_color_detect() == 1:
            bridge_move(0)
            ev3.speaker.beep()
            ev3.speaker.beep()
            break
    bridge_move(180) # Prepare a position to call for head calibration
    wait(600)
    bridge_move(0)
    mbox.send('Calibration')
    mbox.wait()
    bridge_move(-180)
    while True:
        if rail_color_detect() == 1:
            bridge_move(0)
            ev3.speaker.beep()
            ev3.speaker.beep()
            break    
        
def bridge_move(speed): # Speed = 0 to stop the bridge
    if speed == 0:
        left_wheel.hold()
        right_wheel.hold()

    left_wheel.run(speed)
    right_wheel.run(speed)

# Start of the main program            
ev3.speaker.beep()
bridge_position()

ev3.speaker.set_volume(100)
ev3.speaker.say("Scan the color now")
color_seq = []
color_selection()

ev3.speaker.beep()

for i in range(0,4):
    bridge_move(180)

    while True:        
        if rail_color_detect() == color_seq[i]:
            bridge_move(0)
            mbox.send('Pickup')
            mbox.wait()
            
            bridge_move(-180)
            while True:
                if rail_color_detect() == 1:
                    bridge_move(0)
                    break
                        
            mbox.send('Release')
            mbox.wait()
            break

mbox.send('Spin')
mbox.wait()
bridge_move(1500)
wait(1000)
bridge_move(0)
mbox.send('All Done')